/////////////////////////////////////////////////////////////////////////////////
// Company       : 武汉芯路恒科技有限公司
//                 http://xiaomeige.taobao.com
// Web           : http://www.corecourse.cn
// 
// Create Date   : 2019/05/01 00:00:00
// Module Name   : data_scan_ctrl
// Description   : 数据动态扫描模块
// 
// Dependencies  : 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
/////////////////////////////////////////////////////////////////////////////////

module data_scan_ctrl #(
  parameter ROW_DISP_CHAR_NUM = 10 , //1行显示字符个数
  parameter COL_DISP_CHAR_NUM = 8    //显示字符行数
)
(
  input                                                 clk,
  input                                                 reset_n,

  input                                                 scan_en_pulse,
  input     [ROW_DISP_CHAR_NUM*COL_DISP_CHAR_NUM*4-1:0] disp_data,

  output reg[8:0]                                       extr_data_addr,
  output reg[ROW_DISP_CHAR_NUM*COL_DISP_CHAR_NUM-1:0]   update_data_wr,
  output reg[4:0]                                       update_data_addr
);
reg                                           scan_state;
reg                                           extr_data_done;
reg [15:0]                                    addr_cnt;
reg [15:0]                                    addr_cnt_dly1;
reg [15:0]                                    addr_cnt_dly2;
reg [15:0]                                    addr_cnt_dly3;
reg [ROW_DISP_CHAR_NUM*COL_DISP_CHAR_NUM-1:0] extr_data_cs;
reg [ROW_DISP_CHAR_NUM*COL_DISP_CHAR_NUM-1:0] extr_data_cs_dly1;
reg [ROW_DISP_CHAR_NUM*COL_DISP_CHAR_NUM-1:0] extr_data_cs_dly2;
reg [8:0]                                     extr_data_addr_offset;
reg [3:0]                                     extr_data_num;

  always@(posedge clk or negedge reset_n)
  begin
    if(~reset_n)
      scan_state <= 1'b0;
    else if(addr_cnt == (32*ROW_DISP_CHAR_NUM*COL_DISP_CHAR_NUM-1))
      scan_state <= 1'b0;
    else if(scan_en_pulse)
      scan_state <= 1'b1;
    else
      scan_state <= scan_state;
  end

  always@(posedge clk or negedge reset_n)
  begin
    if(~reset_n)
      addr_cnt <= 'd0;
    else if(scan_state == 1'b1)
      addr_cnt <= addr_cnt + 1'b1;
    else
      addr_cnt <= 'd0;
  end

  always@(posedge clk)
  begin
      addr_cnt_dly1 <= addr_cnt;
      addr_cnt_dly2 <= addr_cnt_dly1;
      addr_cnt_dly3 <= addr_cnt_dly2;
  end

  always@(posedge clk or negedge reset_n)
  begin
    if(~reset_n)
      extr_data_done <= 1'b0;
    else if(addr_cnt[4:0] == 5'h1f)
      extr_data_done <= 1'b1;
    else
      extr_data_done <= 1'b0;
  end

  always@(posedge clk or negedge reset_n)
  begin
    if(~reset_n)
      extr_data_cs <= 'd0;
    else if(scan_en_pulse)
      extr_data_cs <= 'd1;
    else if(extr_data_done)
      extr_data_cs <= extr_data_cs << 1'b1;
    else
      extr_data_cs <= extr_data_cs;
  end

  always@(posedge clk)
  begin
      extr_data_cs_dly1 <= extr_data_cs;
      extr_data_cs_dly2 <= extr_data_cs_dly1;
  end

  always@(*)
  begin
    case(extr_data_cs)
      80'h0000_0000_0000_0000_0001: extr_data_num = disp_data[3:0];
      80'h0000_0000_0000_0000_0002: extr_data_num = disp_data[7:4];
      80'h0000_0000_0000_0000_0004: extr_data_num = disp_data[11:8];
      80'h0000_0000_0000_0000_0008: extr_data_num = disp_data[15:12];
      80'h0000_0000_0000_0000_0010: extr_data_num = disp_data[19:16];
      80'h0000_0000_0000_0000_0020: extr_data_num = disp_data[23:20];
      80'h0000_0000_0000_0000_0040: extr_data_num = disp_data[27:24];
      80'h0000_0000_0000_0000_0080: extr_data_num = disp_data[31:28];
      80'h0000_0000_0000_0000_0100: extr_data_num = disp_data[35:32];
      80'h0000_0000_0000_0000_0200: extr_data_num = disp_data[39:36];
      80'h0000_0000_0000_0000_0400: extr_data_num = disp_data[43:40];
      80'h0000_0000_0000_0000_0800: extr_data_num = disp_data[47:44];
      80'h0000_0000_0000_0000_1000: extr_data_num = disp_data[51:48];
      80'h0000_0000_0000_0000_2000: extr_data_num = disp_data[55:52];
      80'h0000_0000_0000_0000_4000: extr_data_num = disp_data[59:56];
      80'h0000_0000_0000_0000_8000: extr_data_num = disp_data[63:60];
      80'h0000_0000_0000_0001_0000: extr_data_num = disp_data[67:64];
      80'h0000_0000_0000_0002_0000: extr_data_num = disp_data[71:68];
      80'h0000_0000_0000_0004_0000: extr_data_num = disp_data[75:72];
      80'h0000_0000_0000_0008_0000: extr_data_num = disp_data[79:76];
      80'h0000_0000_0000_0010_0000: extr_data_num = disp_data[83:80];
      80'h0000_0000_0000_0020_0000: extr_data_num = disp_data[87:84];
      80'h0000_0000_0000_0040_0000: extr_data_num = disp_data[91:88];
      80'h0000_0000_0000_0080_0000: extr_data_num = disp_data[95:92];
      80'h0000_0000_0000_0100_0000: extr_data_num = disp_data[99:96];
      80'h0000_0000_0000_0200_0000: extr_data_num = disp_data[103:100];
      80'h0000_0000_0000_0400_0000: extr_data_num = disp_data[107:104];
      80'h0000_0000_0000_0800_0000: extr_data_num = disp_data[111:108];
      80'h0000_0000_0000_1000_0000: extr_data_num = disp_data[115:112];
      80'h0000_0000_0000_2000_0000: extr_data_num = disp_data[119:116];
      80'h0000_0000_0000_4000_0000: extr_data_num = disp_data[123:120];
      80'h0000_0000_0000_8000_0000: extr_data_num = disp_data[127:124];
      80'h0000_0000_0001_0000_0000: extr_data_num = disp_data[131:128];
      80'h0000_0000_0002_0000_0000: extr_data_num = disp_data[135:132];
      80'h0000_0000_0004_0000_0000: extr_data_num = disp_data[139:136];
      80'h0000_0000_0008_0000_0000: extr_data_num = disp_data[143:140];
      80'h0000_0000_0010_0000_0000: extr_data_num = disp_data[147:144];
      80'h0000_0000_0020_0000_0000: extr_data_num = disp_data[151:148];
      80'h0000_0000_0040_0000_0000: extr_data_num = disp_data[155:152];
      80'h0000_0000_0080_0000_0000: extr_data_num = disp_data[159:156];
      80'h0000_0000_0100_0000_0000: extr_data_num = disp_data[163:160];
      80'h0000_0000_0200_0000_0000: extr_data_num = disp_data[167:164];
      80'h0000_0000_0400_0000_0000: extr_data_num = disp_data[171:168];
      80'h0000_0000_0800_0000_0000: extr_data_num = disp_data[175:172];
      80'h0000_0000_1000_0000_0000: extr_data_num = disp_data[179:176];
      80'h0000_0000_2000_0000_0000: extr_data_num = disp_data[183:180];
      80'h0000_0000_4000_0000_0000: extr_data_num = disp_data[187:184];
      80'h0000_0000_8000_0000_0000: extr_data_num = disp_data[191:188];
      80'h0000_0001_0000_0000_0000: extr_data_num = disp_data[195:192];
      80'h0000_0002_0000_0000_0000: extr_data_num = disp_data[199:196];
      80'h0000_0004_0000_0000_0000: extr_data_num = disp_data[203:200];
      80'h0000_0008_0000_0000_0000: extr_data_num = disp_data[207:204];
      80'h0000_0010_0000_0000_0000: extr_data_num = disp_data[211:208];
      80'h0000_0020_0000_0000_0000: extr_data_num = disp_data[215:212];
      80'h0000_0040_0000_0000_0000: extr_data_num = disp_data[219:216];
      80'h0000_0080_0000_0000_0000: extr_data_num = disp_data[223:220];
      80'h0000_0100_0000_0000_0000: extr_data_num = disp_data[227:224];
      80'h0000_0200_0000_0000_0000: extr_data_num = disp_data[231:228];
      80'h0000_0400_0000_0000_0000: extr_data_num = disp_data[235:232];
      80'h0000_0800_0000_0000_0000: extr_data_num = disp_data[239:236];
      80'h0000_1000_0000_0000_0000: extr_data_num = disp_data[243:240];
      80'h0000_2000_0000_0000_0000: extr_data_num = disp_data[247:244];
      80'h0000_4000_0000_0000_0000: extr_data_num = disp_data[251:248];
      80'h0000_8000_0000_0000_0000: extr_data_num = disp_data[255:252];
      80'h0001_0000_0000_0000_0000: extr_data_num = disp_data[259:256];
      80'h0002_0000_0000_0000_0000: extr_data_num = disp_data[263:260];
      80'h0004_0000_0000_0000_0000: extr_data_num = disp_data[267:264];
      80'h0008_0000_0000_0000_0000: extr_data_num = disp_data[271:268];
      80'h0010_0000_0000_0000_0000: extr_data_num = disp_data[275:272];
      80'h0020_0000_0000_0000_0000: extr_data_num = disp_data[279:276];
      80'h0040_0000_0000_0000_0000: extr_data_num = disp_data[283:280];
      80'h0080_0000_0000_0000_0000: extr_data_num = disp_data[287:284];
      80'h0100_0000_0000_0000_0000: extr_data_num = disp_data[291:288];
      80'h0200_0000_0000_0000_0000: extr_data_num = disp_data[295:292];
      80'h0400_0000_0000_0000_0000: extr_data_num = disp_data[299:296];
      80'h0800_0000_0000_0000_0000: extr_data_num = disp_data[303:300];
      80'h1000_0000_0000_0000_0000: extr_data_num = disp_data[307:304];
      80'h2000_0000_0000_0000_0000: extr_data_num = disp_data[311:308];
      80'h4000_0000_0000_0000_0000: extr_data_num = disp_data[315:312];
      80'h8000_0000_0000_0000_0000: extr_data_num = disp_data[319:316];
      default     : extr_data_num = 4'h0;
    endcase
  end

  always@(posedge clk or negedge reset_n)
  begin
    if(~reset_n)
      extr_data_addr_offset <= 'd0;
    else
      case(extr_data_num)
        4'h0   :extr_data_addr_offset <= 'd0;
        4'h1   :extr_data_addr_offset <= 'd32;
        4'h2   :extr_data_addr_offset <= 'd64;
        4'h3   :extr_data_addr_offset <= 'd96;
        4'h4   :extr_data_addr_offset <= 'd128;
        4'h5   :extr_data_addr_offset <= 'd160;
        4'h6   :extr_data_addr_offset <= 'd192;
        4'h7   :extr_data_addr_offset <= 'd224;
        4'h8   :extr_data_addr_offset <= 'd256;
        4'h9   :extr_data_addr_offset <= 'd288;
        default:extr_data_addr_offset <= 'd0;
      endcase
  end

  always@(posedge clk or negedge reset_n)
  begin
    if(~reset_n)
      extr_data_addr <= 'd0;
    else
      extr_data_addr <= extr_data_addr_offset + addr_cnt_dly2[4:0];
  end

  always@(posedge clk or negedge reset_n)
  begin
    if(~reset_n)
      update_data_addr <= 'd0;
    else
      update_data_addr <= addr_cnt_dly3[4:0];
  end

  always@(posedge clk or negedge reset_n)
  begin
    if(~reset_n)
      update_data_wr <= 'd0;
    else
      update_data_wr <= extr_data_cs_dly2;
  end

endmodule